"""
    Cisco Intersight

    Cisco Intersight is a management platform delivered as a service with embedded analytics for your Cisco and 3rd party IT infrastructure. This platform offers an intelligent level of management that enables IT organizations to analyze, simplify, and automate their environments in more advanced ways than the prior generations of tools. Cisco Intersight provides an integrated and intuitive management experience for resources in the traditional data center as well as at the edge. With flexible deployment options to address complex security needs, getting started with Intersight is quick and easy. Cisco Intersight has deep integration with Cisco UCS and HyperFlex systems allowing for remote deployment, configuration, and ongoing maintenance. The model-based deployment works for a single system in a remote location or hundreds of systems in a data center and enables rapid, standardized configuration and deployment. It also streamlines maintaining those systems whether you are working with small or very large configurations. The Intersight OpenAPI document defines the complete set of properties that are returned in the HTTP response. From that perspective, a client can expect that no additional properties are returned, unless these properties are explicitly defined in the OpenAPI document. However, when a client uses an older version of the Intersight OpenAPI document, the server may send additional properties because the software is more recent than the client. In that case, the client may receive properties that it does not know about. Some generated SDKs perform a strict validation of the HTTP response body against the OpenAPI document.  # noqa: E501

    The version of the OpenAPI document: 1.0.11-18369
    Contact: intersight@cisco.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from intersight.model_utils import (  # noqa: F401
    ApiTypeError,
    ModelComposed,
    ModelNormal,
    ModelSimple,
    cached_property,
    change_keys_js_to_python,
    convert_js_args_to_python_args,
    date,
    datetime,
    file_type,
    none_type,
    validate_get_composed_info,
    set_model_init_error,
    OpenApiModel
)
from intersight.exceptions import ApiAttributeError


def lazy_import():
    from intersight.model.mo_base_complex_type import MoBaseComplexType
    from intersight.model.os_answers import OsAnswers
    from intersight.model.os_install_target import OsInstallTarget
    from intersight.model.os_operating_system_parameters import OsOperatingSystemParameters
    from intersight.model.os_place_holder import OsPlaceHolder
    from intersight.model.os_server_config_all_of import OsServerConfigAllOf
    globals()['MoBaseComplexType'] = MoBaseComplexType
    globals()['OsAnswers'] = OsAnswers
    globals()['OsInstallTarget'] = OsInstallTarget
    globals()['OsOperatingSystemParameters'] = OsOperatingSystemParameters
    globals()['OsPlaceHolder'] = OsPlaceHolder
    globals()['OsServerConfigAllOf'] = OsServerConfigAllOf


class OsServerConfig(ModelComposed):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Attributes:
      allowed_values (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          with a capitalized key describing the allowed value and an allowed
          value. These dicts store the allowed enum values.
      attribute_map (dict): The key is attribute name
          and the value is json key in definition.
      discriminator_value_class_map (dict): A dict to go from the discriminator
          variable value to the discriminator class name.
      validations (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          that stores validations for max_length, min_length, max_items,
          min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
          inclusive_minimum, and regex.
      additional_properties_type (tuple): A tuple of classes accepted
          as additional properties values.
    """

    allowed_values = {
        ('class_id',): {
            'OS.SERVERCONFIG': "os.ServerConfig",
        },
        ('object_type',): {
            'OS.SERVERCONFIG': "os.ServerConfig",
        },
    }

    validations = {
        ('initiator_wwpn',): {
            'regex': {
                'pattern': r'^$|(^([0-9a-fA-F]{2}:){7}[0-9a-fA-F]{2}$)',  # noqa: E501
            },
        },
        ('lun_id',): {
            'inclusive_maximum': 255,
            'inclusive_minimum': 0,
        },
        ('target_wwpn',): {
            'regex': {
                'pattern': r'^$|(^([0-9a-fA-F]{2}:){7}[0-9a-fA-F]{2}$)',  # noqa: E501
            },
        },
        ('vnic_mac',): {
            'regex': {
                'pattern': r'^$|^([0-9A-Fa-f]{2}[:]){5}([0-9A-Fa-f]{2})$',  # noqa: E501
            },
        },
    }

    @cached_property
    def additional_properties_type():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded
        """
        lazy_import()
        return (bool, date, datetime, dict, float, int, list, str, none_type,)  # noqa: E501

    _nullable = True

    @cached_property
    def openapi_types():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded

        Returns
            openapi_types (dict): The key is attribute name
                and the value is attribute type.
        """
        lazy_import()
        return {
            'class_id': (str,),  # noqa: E501
            'object_type': (str,),  # noqa: E501
            'additional_parameters': ([OsPlaceHolder], none_type,),  # noqa: E501
            'answers': (OsAnswers,),  # noqa: E501
            'error_msgs': ([str], none_type,),  # noqa: E501
            'initiator_wwpn': (str,),  # noqa: E501
            'install_target': (str,),  # noqa: E501
            'lun_id': (int,),  # noqa: E501
            'operating_system_parameters': (OsOperatingSystemParameters,),  # noqa: E501
            'processed_install_target': (OsInstallTarget,),  # noqa: E501
            'serial_number': (str,),  # noqa: E501
            'target_iqn': (str,),  # noqa: E501
            'target_wwpn': (str,),  # noqa: E501
            'vnic_mac': (str,),  # noqa: E501
        }

    @cached_property
    def discriminator():
        val = {
        }
        if not val:
            return None
        return {'class_id': val}

    attribute_map = {
        'class_id': 'ClassId',  # noqa: E501
        'object_type': 'ObjectType',  # noqa: E501
        'additional_parameters': 'AdditionalParameters',  # noqa: E501
        'answers': 'Answers',  # noqa: E501
        'error_msgs': 'ErrorMsgs',  # noqa: E501
        'initiator_wwpn': 'InitiatorWwpn',  # noqa: E501
        'install_target': 'InstallTarget',  # noqa: E501
        'lun_id': 'LunId',  # noqa: E501
        'operating_system_parameters': 'OperatingSystemParameters',  # noqa: E501
        'processed_install_target': 'ProcessedInstallTarget',  # noqa: E501
        'serial_number': 'SerialNumber',  # noqa: E501
        'target_iqn': 'TargetIqn',  # noqa: E501
        'target_wwpn': 'TargetWwpn',  # noqa: E501
        'vnic_mac': 'VnicMac',  # noqa: E501
    }

    read_only_vars = {
        'initiator_wwpn',  # noqa: E501
        'install_target',  # noqa: E501
        'lun_id',  # noqa: E501
        'serial_number',  # noqa: E501
        'target_iqn',  # noqa: E501
        'target_wwpn',  # noqa: E501
        'vnic_mac',  # noqa: E501
    }

    @classmethod
    @convert_js_args_to_python_args
    def _from_openapi_data(cls, *args, **kwargs):  # noqa: E501
        """OsServerConfig - a model defined in OpenAPI

        Keyword Args:
            class_id (str): The fully-qualified name of the instantiated, concrete type. This property is used as a discriminator to identify the type of the payload when marshaling and unmarshaling data.. defaults to "os.ServerConfig", must be one of ["os.ServerConfig", ]  # noqa: E501
            object_type (str): The fully-qualified name of the instantiated, concrete type. The value should be the same as the 'ClassId' property.. defaults to "os.ServerConfig", must be one of ["os.ServerConfig", ]  # noqa: E501
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            additional_parameters ([OsPlaceHolder], none_type): [optional]  # noqa: E501
            answers (OsAnswers): [optional]  # noqa: E501
            error_msgs ([str], none_type): [optional]  # noqa: E501
            initiator_wwpn (str): The WWPN Address of the underlying fibre channel interface at the host side used for SAN accesss. Value must be in hexadecimal format xx:xx:xx:xx:xx:xx:xx:xx.  For example, 20:00:D4:C9:3C:35:02:01.. [optional]  # noqa: E501
            install_target (str): The target in which OS installation triggered, the value represented is StorageControllerID follwed by PhysicalDisk SerialNumber in case of Physcial disk or VirtualDriveId for virtual drive.. [optional]  # noqa: E501
            lun_id (int): The Logical Unit Number (LUN) of the install target.. [optional] if omitted the server will use the default value of 0  # noqa: E501
            operating_system_parameters (OsOperatingSystemParameters): [optional]  # noqa: E501
            processed_install_target (OsInstallTarget): [optional]  # noqa: E501
            serial_number (str): The Serial Number of the server.. [optional]  # noqa: E501
            target_iqn (str): IQN (iSCSI qualified name) of Storage iSCSI target. Can be up to 255 characters long and has the following format, iqn.yyyy-mm.naming-authority:unique_name.. [optional]  # noqa: E501
            target_wwpn (str): The WWPN Address of the underlying fibre channel interface at the target used by the storage. Value must be in hexadecimal format xx:xx:xx:xx:xx:xx:xx:xx.  For example, 51:4F:0C:50:14:1F:AF:01.. [optional]  # noqa: E501
            vnic_mac (str): MAC address of the VNIC used as iSCSI initiator interface.. [optional]  # noqa: E501
        """

        class_id = "os.ServerConfig" if len(kwargs.get('class_id','').strip()) == 0 else kwargs.get('class_id', "os.ServerConfig")
        object_type = "os.ServerConfig" if len(kwargs.get('object_type','').strip()) == 0 else kwargs.get('object_type', "os.ServerConfig")
        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        self = super(OpenApiModel, cls).__new__(cls)

        if args:
            for arg in args:
                if isinstance(arg, dict):
                    kwargs.update(arg)
                else:
                    raise ApiTypeError(
                        "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                            args,
                            self.__class__.__name__,
                        ),
                        path_to_item=_path_to_item,
                        valid_classes=(self.__class__,),
                    )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        constant_args = {
            '_check_type': _check_type,
            '_path_to_item': _path_to_item,
            '_spec_property_naming': _spec_property_naming,
            '_configuration': _configuration,
            '_visited_composed_classes': self._visited_composed_classes,
        }
        required_args = {
            'class_id': class_id,
            'object_type': object_type,
        }
        kwargs.update(required_args)
        composed_info = validate_get_composed_info(
            constant_args, kwargs, self)
        self._composed_instances = composed_info[0]
        self._var_name_to_model_instances = composed_info[1]
        self._additional_properties_model_instances = composed_info[2]
        discarded_args = composed_info[3]
        composed_keys = set()
        for instance in self._composed_instances:
            composed_keys.update(set(instance.attribute_map.values()))

        for var_name, var_value in kwargs.items():
            if var_name in discarded_args and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self._additional_properties_model_instances:
                # discard variable.
                continue
            if var_name in composed_keys:
                continue
            setattr(self, var_name, var_value)

        return self


    required_properties = set([
        '_data_store',
        '_check_type',
        '_spec_property_naming',
        '_path_to_item',
        '_configuration',
        '_visited_composed_classes',
        '_composed_instances',
        '_var_name_to_model_instances',
        '_additional_properties_model_instances',
    ])

    @convert_js_args_to_python_args
    def __init__(self, *args, **kwargs):  # noqa: E501
        """OsServerConfig - a model defined in OpenAPI

        Keyword Args:
            class_id (str): The fully-qualified name of the instantiated, concrete type. This property is used as a discriminator to identify the type of the payload when marshaling and unmarshaling data.. defaults to "os.ServerConfig", must be one of ["os.ServerConfig", ]  # noqa: E501
            object_type (str): The fully-qualified name of the instantiated, concrete type. The value should be the same as the 'ClassId' property.. defaults to "os.ServerConfig", must be one of ["os.ServerConfig", ]  # noqa: E501
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            additional_parameters ([OsPlaceHolder], none_type): [optional]  # noqa: E501
            answers (OsAnswers): [optional]  # noqa: E501
            error_msgs ([str], none_type): [optional]  # noqa: E501
            initiator_wwpn (str): The WWPN Address of the underlying fibre channel interface at the host side used for SAN accesss. Value must be in hexadecimal format xx:xx:xx:xx:xx:xx:xx:xx.  For example, 20:00:D4:C9:3C:35:02:01.. [optional]  # noqa: E501
            install_target (str): The target in which OS installation triggered, the value represented is StorageControllerID follwed by PhysicalDisk SerialNumber in case of Physcial disk or VirtualDriveId for virtual drive.. [optional]  # noqa: E501
            lun_id (int): The Logical Unit Number (LUN) of the install target.. [optional] if omitted the server will use the default value of 0  # noqa: E501
            operating_system_parameters (OsOperatingSystemParameters): [optional]  # noqa: E501
            processed_install_target (OsInstallTarget): [optional]  # noqa: E501
            serial_number (str): The Serial Number of the server.. [optional]  # noqa: E501
            target_iqn (str): IQN (iSCSI qualified name) of Storage iSCSI target. Can be up to 255 characters long and has the following format, iqn.yyyy-mm.naming-authority:unique_name.. [optional]  # noqa: E501
            target_wwpn (str): The WWPN Address of the underlying fibre channel interface at the target used by the storage. Value must be in hexadecimal format xx:xx:xx:xx:xx:xx:xx:xx.  For example, 51:4F:0C:50:14:1F:AF:01.. [optional]  # noqa: E501
            vnic_mac (str): MAC address of the VNIC used as iSCSI initiator interface.. [optional]  # noqa: E501
        """

        class_id = "os.ServerConfig" if len(kwargs.get('class_id','').strip()) == 0 else kwargs.get('class_id', "os.ServerConfig")
        object_type = "os.ServerConfig" if len(kwargs.get('object_type','').strip()) == 0 else kwargs.get('object_type', "os.ServerConfig")
        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        if args:
            for arg in args:
                if isinstance(arg, dict):
                    kwargs.update(arg)
                else:
                    raise ApiTypeError(
                        "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                            args,
                            self.__class__.__name__,
                        ),
                        path_to_item=_path_to_item,
                        valid_classes=(self.__class__,),
                    )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        constant_args = {
            '_check_type': _check_type,
            '_path_to_item': _path_to_item,
            '_spec_property_naming': _spec_property_naming,
            '_configuration': _configuration,
            '_visited_composed_classes': self._visited_composed_classes,
        }
        required_args = {
            'class_id': class_id,
            'object_type': object_type,
        }
        kwargs.update(required_args)
        composed_info = validate_get_composed_info(
            constant_args, kwargs, self)
        self._composed_instances = composed_info[0]
        self._var_name_to_model_instances = composed_info[1]
        self._additional_properties_model_instances = composed_info[2]
        discarded_args = composed_info[3]

        for var_name, var_value in kwargs.items():
            if var_name in self.read_only_vars:
                set_model_init_error(f"Warning : {var_name} is omitted from request payload as it is a read-only property")
                continue
            if var_name in discarded_args and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self._additional_properties_model_instances:
                # discard variable.
                continue
            setattr(self, var_name, var_value)

    @cached_property
    def _composed_schemas():
        # we need this here to make our import statements work
        # we must store _composed_schemas in here so the code is only run
        # when we invoke this method. If we kept this at the class
        # level we would get an error because the class level
        # code would be run when this module is imported, and these composed
        # classes don't exist yet because their module has not finished
        # loading
        lazy_import()
        return {
          'anyOf': [
          ],
          'allOf': [
              MoBaseComplexType,
              OsServerConfigAllOf,
          ],
          'oneOf': [
          ],
        }
