"""
    Cisco Intersight

    Cisco Intersight is a management platform delivered as a service with embedded analytics for your Cisco and 3rd party IT infrastructure. This platform offers an intelligent level of management that enables IT organizations to analyze, simplify, and automate their environments in more advanced ways than the prior generations of tools. Cisco Intersight provides an integrated and intuitive management experience for resources in the traditional data center as well as at the edge. With flexible deployment options to address complex security needs, getting started with Intersight is quick and easy. Cisco Intersight has deep integration with Cisco UCS and HyperFlex systems allowing for remote deployment, configuration, and ongoing maintenance. The model-based deployment works for a single system in a remote location or hundreds of systems in a data center and enables rapid, standardized configuration and deployment. It also streamlines maintaining those systems whether you are working with small or very large configurations. The Intersight OpenAPI document defines the complete set of properties that are returned in the HTTP response. From that perspective, a client can expect that no additional properties are returned, unless these properties are explicitly defined in the OpenAPI document. However, when a client uses an older version of the Intersight OpenAPI document, the server may send additional properties because the software is more recent than the client. In that case, the client may receive properties that it does not know about. Some generated SDKs perform a strict validation of the HTTP response body against the OpenAPI document.  # noqa: E501

    The version of the OpenAPI document: 1.0.11-18369
    Contact: intersight@cisco.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from intersight.model_utils import (  # noqa: F401
    ApiTypeError,
    ModelComposed,
    ModelNormal,
    ModelSimple,
    cached_property,
    change_keys_js_to_python,
    convert_js_args_to_python_args,
    date,
    datetime,
    file_type,
    none_type,
    validate_get_composed_info,
    set_model_init_error,
    OpenApiModel
)
from intersight.exceptions import ApiAttributeError


def lazy_import():
    from intersight.model.fcpool_lease_relationship import FcpoolLeaseRelationship
    from intersight.model.motemplate_action_entry import MotemplateActionEntry
    from intersight.model.motemplate_sync_error import MotemplateSyncError
    from intersight.model.policy_abstract_config_profile_relationship import PolicyAbstractConfigProfileRelationship
    from intersight.model.vnic_fc_if_relationship import VnicFcIfRelationship
    from intersight.model.vnic_placement_settings import VnicPlacementSettings
    from intersight.model.vnic_san_connectivity_policy_relationship import VnicSanConnectivityPolicyRelationship
    from intersight.model.vnic_vhba_template_relationship import VnicVhbaTemplateRelationship
    globals()['FcpoolLeaseRelationship'] = FcpoolLeaseRelationship
    globals()['MotemplateActionEntry'] = MotemplateActionEntry
    globals()['MotemplateSyncError'] = MotemplateSyncError
    globals()['PolicyAbstractConfigProfileRelationship'] = PolicyAbstractConfigProfileRelationship
    globals()['VnicFcIfRelationship'] = VnicFcIfRelationship
    globals()['VnicPlacementSettings'] = VnicPlacementSettings
    globals()['VnicSanConnectivityPolicyRelationship'] = VnicSanConnectivityPolicyRelationship
    globals()['VnicVhbaTemplateRelationship'] = VnicVhbaTemplateRelationship


class VnicFcIfAllOf(ModelNormal):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Attributes:
      allowed_values (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          with a capitalized key describing the allowed value and an allowed
          value. These dicts store the allowed enum values.
      attribute_map (dict): The key is attribute name
          and the value is json key in definition.
      discriminator_value_class_map (dict): A dict to go from the discriminator
          variable value to the discriminator class name.
      validations (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          that stores validations for max_length, min_length, max_items,
          min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
          inclusive_minimum, and regex.
      additional_properties_type (tuple): A tuple of classes accepted
          as additional properties values.
    """

    allowed_values = {
        ('class_id',): {
            'VNIC.FCIF': "vnic.FcIf",
        },
        ('object_type',): {
            'VNIC.FCIF': "vnic.FcIf",
        },
        ('template_sync_status',): {
            'NONE': "None",
            'OK': "OK",
            'SCHEDULED': "Scheduled",
            'INPROGRESS': "InProgress",
            'OUTOFSYNC': "OutOfSync",
        },
        ('wwpn_address_type',): {
            'POOL': "POOL",
            'STATIC': "STATIC",
        },
    }

    validations = {
        ('name',): {
            'max_length': 31,
            'regex': {
                'pattern': r'^[a-zA-Z0-9\-\._:]+$',  # noqa: E501
            },
        },
        ('static_wwpn_address',): {
            'regex': {
                'pattern': r'^$|((^20|5[0-9a-fA-F]{1}):([0-9a-fA-F]{2}:){6}([0-9a-fA-F]{2})$)',  # noqa: E501
            },
        },
        ('wwpn',): {
            'regex': {
                'pattern': r'^$|((^20|5[0-9a-fA-F]{1}):([0-9a-fA-F]{2}:){6}([0-9a-fA-F]{2})$)',  # noqa: E501
            },
        },
    }

    @cached_property
    def additional_properties_type():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded
        """
        lazy_import()
        return (bool, date, datetime, dict, float, int, list, str, none_type,)  # noqa: E501

    _nullable = False

    @cached_property
    def openapi_types():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded

        Returns
            openapi_types (dict): The key is attribute name
                and the value is attribute type.
        """
        lazy_import()
        return {
            'class_id': (str,),  # noqa: E501
            'object_type': (str,),  # noqa: E501
            'name': (str,),  # noqa: E501
            'order': (int,),  # noqa: E501
            'overridden_list': ([str], none_type,),  # noqa: E501
            'placement': (VnicPlacementSettings,),  # noqa: E501
            'static_wwpn_address': (str,),  # noqa: E501
            'template_actions': ([MotemplateActionEntry], none_type,),  # noqa: E501
            'template_sync_errors': ([MotemplateSyncError], none_type,),  # noqa: E501
            'template_sync_status': (str,),  # noqa: E501
            'vif_id': (int,),  # noqa: E501
            'wwpn': (str,),  # noqa: E501
            'wwpn_address_type': (str,),  # noqa: E501
            'profile': (PolicyAbstractConfigProfileRelationship,),  # noqa: E501
            'san_connectivity_policy': (VnicSanConnectivityPolicyRelationship,),  # noqa: E501
            'scp_vhba': (VnicFcIfRelationship,),  # noqa: E501
            'sp_vhbas': ([VnicFcIfRelationship], none_type,),  # noqa: E501
            'src_template': (VnicVhbaTemplateRelationship,),  # noqa: E501
            'wwpn_lease': (FcpoolLeaseRelationship,),  # noqa: E501
        }

    @cached_property
    def discriminator():
        return None


    attribute_map = {
        'class_id': 'ClassId',  # noqa: E501
        'object_type': 'ObjectType',  # noqa: E501
        'name': 'Name',  # noqa: E501
        'order': 'Order',  # noqa: E501
        'overridden_list': 'OverriddenList',  # noqa: E501
        'placement': 'Placement',  # noqa: E501
        'static_wwpn_address': 'StaticWwpnAddress',  # noqa: E501
        'template_actions': 'TemplateActions',  # noqa: E501
        'template_sync_errors': 'TemplateSyncErrors',  # noqa: E501
        'template_sync_status': 'TemplateSyncStatus',  # noqa: E501
        'vif_id': 'VifId',  # noqa: E501
        'wwpn': 'Wwpn',  # noqa: E501
        'wwpn_address_type': 'WwpnAddressType',  # noqa: E501
        'profile': 'Profile',  # noqa: E501
        'san_connectivity_policy': 'SanConnectivityPolicy',  # noqa: E501
        'scp_vhba': 'ScpVhba',  # noqa: E501
        'sp_vhbas': 'SpVhbas',  # noqa: E501
        'src_template': 'SrcTemplate',  # noqa: E501
        'wwpn_lease': 'WwpnLease',  # noqa: E501
    }

    read_only_vars = {
        'template_sync_status',  # noqa: E501
        'vif_id',  # noqa: E501
        'wwpn',  # noqa: E501
        'sp_vhbas',  # noqa: E501
    }

    _composed_schemas = {}

    @classmethod
    @convert_js_args_to_python_args
    def _from_openapi_data(cls, *args, **kwargs):  # noqa: E501
        """VnicFcIfAllOf - a model defined in OpenAPI

        Args:

        Keyword Args:
            class_id (str): The fully-qualified name of the instantiated, concrete type. This property is used as a discriminator to identify the type of the payload when marshaling and unmarshaling data.. defaults to "vnic.FcIf", must be one of ["vnic.FcIf", ]  # noqa: E501
            object_type (str): The fully-qualified name of the instantiated, concrete type. The value should be the same as the 'ClassId' property.. defaults to "vnic.FcIf", must be one of ["vnic.FcIf", ]  # noqa: E501
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            name (str): Name of the virtual fibre channel interface.. [optional]  # noqa: E501
            order (int): The order in which the virtual interface is brought up. The order assigned to an interface should be unique for all the Ethernet and Fibre-Channel interfaces on each PCI link on a VIC adapter. The order should start from zero with no overlaps. The maximum value of PCI order is limited by the number of virtual interfaces (Ethernet and Fibre-Channel) on each PCI link on a VIC adapter. All VIC adapters have a single PCI link except VIC 1340, VIC 1380 and VIC 1385 which have two.. [optional]  # noqa: E501
            overridden_list ([str], none_type): [optional]  # noqa: E501
            placement (VnicPlacementSettings): [optional]  # noqa: E501
            static_wwpn_address (str): The WWPN address must be in hexadecimal format xx:xx:xx:xx:xx:xx:xx:xx. Allowed ranges are 20:00:00:00:00:00:00:00 to 20:FF:FF:FF:FF:FF:FF:FF or from 50:00:00:00:00:00:00:00 to 5F:FF:FF:FF:FF:FF:FF:FF. To ensure uniqueness of WWN's in the SAN fabric, you are strongly encouraged to use the WWN prefix - 20:00:00:25:B5:xx:xx:xx.. [optional]  # noqa: E501
            template_actions ([MotemplateActionEntry], none_type): [optional]  # noqa: E501
            template_sync_errors ([MotemplateSyncError], none_type): [optional]  # noqa: E501
            template_sync_status (str): The sync status of the current MO wrt the attached Template MO. * `None` - The Enum value represents that the object is not attached to any template. * `OK` - The Enum value represents that the object values are in sync with attached template. * `Scheduled` - The Enum value represents that the object sync from attached template is scheduled from template. * `InProgress` - The Enum value represents that the object sync with the attached template is in progress. * `OutOfSync` - The Enum value represents that the object values are not in sync with attached template.. [optional] if omitted the server will use the default value of "None"  # noqa: E501
            vif_id (int): This should be the same as the channel number of the vfc created on switch in order to set up the data path. The property is applicable only for FI attached servers where a vfc is created on the switch for every vHBA.. [optional]  # noqa: E501
            wwpn (str): The WWPN address that is assigned to the vHBA based on the wwn pool that has been assigned to the SAN Connectivity Policy.. [optional]  # noqa: E501
            wwpn_address_type (str): Type of allocation selected to assign a WWPN address to the vhba. * `POOL` - The user selects a pool from which the mac/wwn address will be leased for the Virtual Interface. * `STATIC` - The user assigns a static mac/wwn address for the Virtual Interface.. [optional] if omitted the server will use the default value of "POOL"  # noqa: E501
            profile (PolicyAbstractConfigProfileRelationship): [optional]  # noqa: E501
            san_connectivity_policy (VnicSanConnectivityPolicyRelationship): [optional]  # noqa: E501
            scp_vhba (VnicFcIfRelationship): [optional]  # noqa: E501
            sp_vhbas ([VnicFcIfRelationship], none_type): An array of relationships to vnicFcIf resources.. [optional]  # noqa: E501
            src_template (VnicVhbaTemplateRelationship): [optional]  # noqa: E501
            wwpn_lease (FcpoolLeaseRelationship): [optional]  # noqa: E501
        """

        class_id = "vnic.FcIf" if len(kwargs.get('class_id','').strip()) == 0 else kwargs.get('class_id', "vnic.FcIf")
        object_type = "vnic.FcIf" if len(kwargs.get('object_type','').strip()) == 0 else kwargs.get('object_type', "vnic.FcIf")
        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', True)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        self = super(OpenApiModel, cls).__new__(cls)

        if args:
            for arg in args:
                if isinstance(arg, dict):
                    kwargs.update(arg)
                else:
                    raise ApiTypeError(
                        "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                            args,
                            self.__class__.__name__,
                        ),
                        path_to_item=_path_to_item,
                        valid_classes=(self.__class__,),
                    )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        self.class_id = class_id
        self.object_type = object_type
        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)
        return self

    required_properties = set([
        '_data_store',
        '_check_type',
        '_spec_property_naming',
        '_path_to_item',
        '_configuration',
        '_visited_composed_classes',
    ])

    @convert_js_args_to_python_args
    def __init__(self, *args, **kwargs):  # noqa: E501
        """VnicFcIfAllOf - a model defined in OpenAPI

        Args:

        Keyword Args:
            class_id (str): The fully-qualified name of the instantiated, concrete type. This property is used as a discriminator to identify the type of the payload when marshaling and unmarshaling data.. defaults to "vnic.FcIf", must be one of ["vnic.FcIf", ]  # noqa: E501
            object_type (str): The fully-qualified name of the instantiated, concrete type. The value should be the same as the 'ClassId' property.. defaults to "vnic.FcIf", must be one of ["vnic.FcIf", ]  # noqa: E501
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            name (str): Name of the virtual fibre channel interface.. [optional]  # noqa: E501
            order (int): The order in which the virtual interface is brought up. The order assigned to an interface should be unique for all the Ethernet and Fibre-Channel interfaces on each PCI link on a VIC adapter. The order should start from zero with no overlaps. The maximum value of PCI order is limited by the number of virtual interfaces (Ethernet and Fibre-Channel) on each PCI link on a VIC adapter. All VIC adapters have a single PCI link except VIC 1340, VIC 1380 and VIC 1385 which have two.. [optional]  # noqa: E501
            overridden_list ([str], none_type): [optional]  # noqa: E501
            placement (VnicPlacementSettings): [optional]  # noqa: E501
            static_wwpn_address (str): The WWPN address must be in hexadecimal format xx:xx:xx:xx:xx:xx:xx:xx. Allowed ranges are 20:00:00:00:00:00:00:00 to 20:FF:FF:FF:FF:FF:FF:FF or from 50:00:00:00:00:00:00:00 to 5F:FF:FF:FF:FF:FF:FF:FF. To ensure uniqueness of WWN's in the SAN fabric, you are strongly encouraged to use the WWN prefix - 20:00:00:25:B5:xx:xx:xx.. [optional]  # noqa: E501
            template_actions ([MotemplateActionEntry], none_type): [optional]  # noqa: E501
            template_sync_errors ([MotemplateSyncError], none_type): [optional]  # noqa: E501
            template_sync_status (str): The sync status of the current MO wrt the attached Template MO. * `None` - The Enum value represents that the object is not attached to any template. * `OK` - The Enum value represents that the object values are in sync with attached template. * `Scheduled` - The Enum value represents that the object sync from attached template is scheduled from template. * `InProgress` - The Enum value represents that the object sync with the attached template is in progress. * `OutOfSync` - The Enum value represents that the object values are not in sync with attached template.. [optional] if omitted the server will use the default value of "None"  # noqa: E501
            vif_id (int): This should be the same as the channel number of the vfc created on switch in order to set up the data path. The property is applicable only for FI attached servers where a vfc is created on the switch for every vHBA.. [optional]  # noqa: E501
            wwpn (str): The WWPN address that is assigned to the vHBA based on the wwn pool that has been assigned to the SAN Connectivity Policy.. [optional]  # noqa: E501
            wwpn_address_type (str): Type of allocation selected to assign a WWPN address to the vhba. * `POOL` - The user selects a pool from which the mac/wwn address will be leased for the Virtual Interface. * `STATIC` - The user assigns a static mac/wwn address for the Virtual Interface.. [optional] if omitted the server will use the default value of "POOL"  # noqa: E501
            profile (PolicyAbstractConfigProfileRelationship): [optional]  # noqa: E501
            san_connectivity_policy (VnicSanConnectivityPolicyRelationship): [optional]  # noqa: E501
            scp_vhba (VnicFcIfRelationship): [optional]  # noqa: E501
            sp_vhbas ([VnicFcIfRelationship], none_type): An array of relationships to vnicFcIf resources.. [optional]  # noqa: E501
            src_template (VnicVhbaTemplateRelationship): [optional]  # noqa: E501
            wwpn_lease (FcpoolLeaseRelationship): [optional]  # noqa: E501
        """

        class_id = "vnic.FcIf" if len(kwargs.get('class_id','').strip()) == 0 else kwargs.get('class_id', "vnic.FcIf")
        object_type = "vnic.FcIf" if len(kwargs.get('object_type','').strip()) == 0 else kwargs.get('object_type', "vnic.FcIf")
        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        if args:
            for arg in args:
                if isinstance(arg, dict):
                    kwargs.update(arg)
                else:
                    raise ApiTypeError(
                        "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                            args,
                            self.__class__.__name__,
                        ),
                        path_to_item=_path_to_item,
                        valid_classes=(self.__class__,),
                    )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        self.class_id = class_id
        self.object_type = object_type
        for var_name, var_value in kwargs.items():
            if var_name in self.read_only_vars:
                set_model_init_error(f"Warning : {var_name} is omitted from request payload as it is a read-only property")
                continue
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)
